# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.1
#      Allows shutdown/restart/suspend/hibernation of your SqueezeCenter server
#      hardware via your SqueezeBox's IR remote or via a SBC
#
#    Version 7.3-RC8-20090505
#
#    Copyright (C) 2008 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin: http://adrianonline.wordpress.com/
#                                                   http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    CLI.pm -- helper routines for srvrpowerctrl CLI processing
#

package Plugins::SrvrPowerCtrl::CLI;

use base qw(Slim::Plugin::Base);
use strict;

use Slim::Utils::Strings qw(string);

#Global Variables..
use Plugins::SrvrPowerCtrl::Settings ();
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;

sub cliGetStatus {
	my ($request) = @_;

	$request->setStatusProcessing();
	$request->addResult('_action', 'enabled');
	$request->setStatusDone();
}

sub cliGetActions {
	my ($request) = @_;
	my @actions = ();
	my $actionlist;

	$request->setStatusProcessing();

	foreach my $item (@{$g{aActions}}) {
		if ($item->{menuindex} ge 0) {
			push (@actions, $item->{'action'});

		}
	}

	$actionlist = join(':', @actions);
	$request->addResult('_action', $actionlist);

	$request->setStatusDone();
}

sub cliSetBlock {
	my ($request) = @_;
	my $client = $request->client();
	my $action = $request->getParam( '_action');
	my $blockowner = $request->getParam('_switchclient');
	my $message = $request->getParam( '_message');

	if ( !defined($blockowner) ) {
		$blockowner = 'viacli';
	}

	$g{log}->debug("Attempting to set block via CLI: caller: $blockowner, message: $message");
	my $blockcode = Plugins::SrvrPowerCtrl::Plugin::blockAction($client, 'set', $blockowner, $message);

	if ($blockcode >= 0) {
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusDone();
		Plugins::SrvrPowerCtrl::Util::DisplayPlayerMessage(undef, $message, 10);
		return;
	} else {
		$g{log}->error( "BadParam: $action");
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusBadParams();
		$request->setStatusDone();
		return;
	}

}

sub cliClearBlock {
	my ($request) = @_;
	my $client = $request->client();
	my $action = $request->getParam( '_action');
	my $blockowner = $request->getParam('_switchclient');
	my $message = $request->getParam( '_message');

	if ( !defined($blockowner) ) {
		$blockowner = 'viacli';
	}

	$g{log}->debug("Attempting to clear block via CLI: caller: $blockowner, message: $message");
	my $blockcode = Plugins::SrvrPowerCtrl::Plugin::blockAction($client, 'clear', $blockowner, $message);
	#A zero result indicates that the last block has been clearned
	if ($blockcode >= 0) {
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusDone();
		Plugins::SrvrPowerCtrl::Util::DisplayPlayerMessage(undef, $message, 10);
		return;
	} else {  #A -1 result indicates error..
		$g{log}->error( "BadParam: $action");
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusBadParams();
		#$request->setStatusDone();
		return;
	}

}

sub cliListBlock {
	my ($request) = @_;
	my $n;

	#this call will clear the array if it's empty..
	if (! Plugins::SrvrPowerCtrl::Block::IsBlocked() ) {
		$request->addResult('_block', 'no_blocks'  );
	} else {
		$n = 0;
		foreach my $element (@{$g{aBlockAction}}) {
			if ( defined($element->{'caller'}) ) {
				$request->addResult('_block' . $n, "$element->{'blockcode'}|$element->{'caller'}|$element->{'reason'}"  );
				$n++;
			}
		}
	}
	$request->setStatusDone();
	return;
}

# -----------------------------------------------------------------------------
# This is the SC-CLI extension offered by the plugin.  Called by CLI, SBC, etc.

sub pluginCLI {
	my ($request) = @_;
	my $client;
	my $curclient;
	my $item;
	my $action = lc($request->getParam( '_action'));
	my $message = $request->getParam( '_message');
	my $switchclient = $request->getParam('_switchclient');
	my $fromJive = $request->getParam('_fromjive');
	my $timeoutmsg;

	$client = $request->client();

	if ( defined($client) ) {
		$g{log}->debug( " Client == " . $client->name() );
	}
	$g{log}->debug( " Action == $action");
	if ( defined($message)) {
		$g{log}->debug( "Message == $message");
	}
	if ( defined($switchclient)) {
		$g{log}->debug( "Switchclient == " . $switchclient );
	}
	if ( defined($fromJive)) {
		$g{log}->debug( "Fromjive == " . $fromJive );
	}


	# If an action is already pending, then this is a request to cancel...
	if ($g{tPendingActionTimer}) {
		return CancelRequest($request, $client);
	}

	# Check that this is a valid request
	if( $request->isNotCommand( [['srvrpowerctrl']])) {
		$g{log}->error( "BadDispatch!");
		$request->setStatusBadDispatch();
		return;
	}

	if ($action eq 'status'){
		#Request is just asking for confirmation that SrvrPowerCtrl is enabled..
		return cliGetStatus($request);
	} elsif ($action eq 'actions'){
		#Request is just asking for a list of the enabled actions..
		return cliGetActions($request);
	} elsif ($action eq 'setblock') {
		#check for block/clear requests..
		return cliSetBlock($request);
	} elsif ($action eq 'clearblock') {
		return cliClearBlock($request);
	} elsif ($action eq 'listblock') {
		return cliListBlock($request);
	}

	# Check for other allowed action items..
	$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
	if (!defined($item)) {
		$g{log}->error( "BadParam: $action");
		$request->addResult('_badparam' . "BadParam: $action"  );
		$request->setStatusBadParams();
		return;
	}

	#OK, this is a valid request..let's proceed..
	$request->setStatusProcessing();


	#if (defined($switchclient) && !defined($fromJive)) {
	if (defined($switchclient)) {
		#Make this a minimum time-out..no opportunity to cancel anyway..
		$item->{cancelwait} = 3;
		if ($switchclient eq 'sleep+hold') {
			#undefine this param...it's not really for this purpose anyway..
			$switchclient = undef;
		} else {
			#Is this a non-Jive CLI request to switch to SN?  If so, then $switchclient is a MAC address..
			#$g{log}->debug( "Switchclient == $switchclient");
			$request->clientid($switchclient);
		}
	}

	#cue up the action..
	if (!Plugins::SrvrPowerCtrl::Plugin::prepareAction($client, $item, 0, $fromJive)) {
		$request->addResult('_action', "$action is blocked.\n");
		$request->setStatusDone();
		return;
	}

	$request->addResult('_action', $action . " in " . $item->{cancelwait} . " seconds.\n");
	$request->setStatusDone();
}


sub CancelRequest {
	my ($request, $client) = @_;
	my $message;

	Plugins::SrvrPowerCtrl::Plugin::cancelAction($client);

	#restore our jive menu..
	#$g{log}->debug("Restoring our jive menu");
	#&addJiveMenu();

	$request->setStatusDone();
	return;
}
