# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.1
#      Allows shutdown/restart/suspend/hibernation of your SqueezeCenter server
#      hardware via your SqueezeBox's IR remote or via a SBC
#
#    Version 7.3-RC8-20090505
#
#    Copyright (C) 2008 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin: http://adrianonline.wordpress.com/
#                                                   http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Watchdog.pm -- watchdog routines..support for shutting down server if players have been idle
#                   for a user-settable time period..
#


package Plugins::SrvrPowerCtrl::Watchdog;

use base qw(Slim::Plugin::Base);
use Slim::Utils::Strings qw(string);
use strict;

use Slim::Utils::DateTime;
use Slim::Utils::Validate;

#Global Variables..
use Plugins::SrvrPowerCtrl::Settings ();
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;

my $nWatchdogActionDelay = 5;	#number of seconds to wait before immediate action..


sub ActivateWatchdogs {

	# Activate the OnWakeup watchdog..
	if ($g{prefs}->bUseOnWakeupWatchdog) {
		ActivateOnWakeupWatchdog($g{prefs}->bUseOnWakeupWatchdog);
	}

	# Activate the SleepPlayDone watchdog..
	if ($g{prefs}->bUseSleepEndWatchdog) {
		ActivateSleepEndWatchdog($g{prefs}->bUseSleepEndWatchdog);
	}

	# Activate the end-of-day idle-player watchdog..
	if ($g{prefs}->bUseEODWatchdog) {
		ActivateEODWatchdog($g{prefs}->bUseEODWatchdog);
	}

	# Activate the idle-player watchdog..
	if ($g{prefs}->bUseIdleWatchdog) {
		ActivateIdleWatchdog($g{prefs}->bUseIdleWatchdog);
	}

}

my $nIdlePlayersTimeCount = 0;

sub ActivateIdleWatchdog {
	my $bEnable = shift;

	#reset the count..
	$nIdlePlayersTimeCount = 0;

	return $bEnable;
}

sub IdleWatchdog {
	my $nCurTime;

	my $item;
	my @clients;
	my $curclient;
	my $action;
	my $nTimeDelay;

	#$g{log}->debug("Checking to see if scanner is running & if players are idle..");
	if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersPlaying() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
		$g{log}->debug("Not idle condition..");
		#Not idle...reset the timer to check again..
		$nIdlePlayersTimeCount = 0;
		return 0;
	}

	#We're idle..increment the idle count..
	#$g{log}->debug("Players are idle..");
	$nIdlePlayersTimeCount++;

	#Has the idle time-out period been exceeded?
	if ( $nIdlePlayersTimeCount >= $g{prefs}->nIdleWatchdogTimeout ) {
		$g{log}->debug("Players are idle and idle timeout of $nIdlePlayersTimeCount has elapsed..");

		$action = $g{prefs}->szIdleWatchdogAction;

		#This is just for testing...not a useful watchdog action..
		if ($action eq 'plugin_menu') {
			@clients = Slim::Player::Client::clients();
			foreach $curclient (@clients) {
				Plugins::SrvrPowerCtrl::Plugin::oursleepButtonHandler( $curclient );
			}
		return;
		}

		$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
		if (!defined($item)) {
			$g{log}->error( "Bad IdleWatchdog Action: $action");
			return;
		}

		#restart the countdown for our eventual return from suspend/hibernate/SN..
		$nIdlePlayersTimeCount = 0;

		#cue up the requested action..
		$g{tPendingActionTimer} = Slim::Utils::Timers::setTimer( undef, time() + $nWatchdogActionDelay, \&Plugins::SrvrPowerCtrl::Plugin::performAction, ( $item ) );

	} else {
		$g{log}->debug( $g{prefs}->nIdleWatchdogTimeout - $nIdlePlayersTimeCount . " minutes left..");
	}
	return 1;
}


sub TimeStrToSeconds {
	my $szTime = shift;
	my $seconds;

	if (  ( !defined($szTime) ) ||
	      ( !defined(Slim::Utils::Validate::isTime($szTime)) )
		) {
		return -1;
	}

	#Slim::Utils::DateTime::hasAmPm

	$seconds = Slim::Utils::DateTime::prettyTimeToSecs($szTime);

	if (!$seconds) {
		return -1;
	}

	return $seconds;
}

sub IsInEOD {
	my $nCurTime = shift || time();
	my $nEODStartSeconds;
	my $nEODEndSeconds;
	my $nNowSeconds;

	if (!$g{prefs}->bUseEODWatchdog) {
		return 0;
	}

	#Get the seconds since midnight..
	my ($lt_sec,$lt_min,$lt_hour) = localtime($nCurTime);
	$nNowSeconds = $lt_sec + ($lt_min * 60) + ($lt_hour * 3600);

	#calculate the nEODStartSeconds..
	$nEODStartSeconds =  TimeStrToSeconds($g{prefs}->szEODWatchdogStartTime);
	if ($nEODStartSeconds == -1) {
		return -1;
	}

	#calculate the nEODEndSeconds..
	$nEODEndSeconds =  TimeStrToSeconds($g{prefs}->szEODWatchdogEndTime);
	if ($nEODEndSeconds == -1) {
		return -1;
	}


	#if we start before midnight and end tomorrow...
	if ( ($nEODStartSeconds > $nEODEndSeconds) && (($nNowSeconds >= $nEODStartSeconds) || ($nNowSeconds <= $nEODEndSeconds)) ) {
		return 1;
	} elsif ( ($nNowSeconds >= $nEODStartSeconds) && ($nNowSeconds <= $nEODEndSeconds) ) {
		return 1;
	}

	return 0;
}

sub TimeToStartOfEOD {
	my $nCurTime = shift || time();
	my ($lt_sec,$lt_min,$lt_hour) = localtime($nCurTime);
	my $nNowSeconds;
	my $nEODStartSeconds;

	#get the seconds since last midnight..
	$nNowSeconds = $lt_sec + ($lt_min * 60) + ($lt_hour * 3600);

	#calculate the nEODStartSeconds..
	$nEODStartSeconds =  TimeStrToSeconds($g{prefs}->szEODWatchdogStartTime);
	if ($nEODStartSeconds == -1) {
		return -1;
	}

	#if we are past start of EOD today..
	if ($nNowSeconds > $nEODStartSeconds) {
		return 86400 - $nNowSeconds + $nEODStartSeconds;
	} else {
		return $nEODStartSeconds - $nNowSeconds;
	}

	return 0;
}

my $bSkipEODCheck = 0;
my $nEODPlayersTimeCount = 0;

sub ActivateEODWatchdog {
	my $bEnable = shift;

	$bSkipEODCheck = ( IsInEOD() && (!$g{nEODWatchdogTimeout}) ) ? 1 : 0;

	if ($bSkipEODCheck) {
		$g{log}->debug("Skipping EOD checks because IsInEOD and bSkipEODCheck == " . $bSkipEODCheck );
	} elsif ($bEnable){
		$nEODPlayersTimeCount = 0;
	}

	return $bEnable;
}

sub EODWatchdog {
	my $nCurTime = shift;

	my $item;
	my @clients;
	my $curclient;
	my $action;
	my $nTimeDelay;

	#if we have an idle period to check..
	if ($g{prefs}->nEODWatchdogTimeout) {
		if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersPlaying() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
			#Not idle...reset the timer to check again..
			$nEODPlayersTimeCount = 0;
			return 0;
		}
	} else {
		#No idle period..skip players playing check..
		if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
			#Not idle...reset the timer to check again..
			$nEODPlayersTimeCount = 0;
			return 0;
		}
	}

	#OK, we're idle..increment the idle count..
	#$g{log}->debug("Players are idle..");
	$nEODPlayersTimeCount++;

	#Have we exceeded the allowed timeout?
	if ( $nEODPlayersTimeCount > $g{prefs}->nEODWatchdogTimeout ) {
		$g{log}->debug("Players are idle and idle timeout of $nEODPlayersTimeCount has elapsed..");

		$action = $g{prefs}->szEODWatchdogAction;

		#This is just for testing...not a useful watchdog action..
		if ($action eq 'plugin_menu') {
			@clients = Slim::Player::Client::clients();
			foreach $curclient (@clients) {
				Plugins::SrvrPowerCtrl::Plugin::oursleepButtonHandler( $curclient );
			}
		return;
		}

		#Alternate EOD action command takes precedence..
		if ( defined($g{prefs}->szEODWatchdog_cmd) && length($g{prefs}->szEODWatchdog_cmd) > 0 ) {
			$action = 'specialcmd';
			$item =  ( {	action					=> $action,
							menuindex				=> 99,
							menutext				=> string('PLUGIN_SRVRPOWERCTRL_SPECIALCMD'),
							message					=> string('PLUGIN_SRVRPOWERCTRL_SPECIALCMD_MSG'),
							dispblock				=> 1,
							getcommandargcoderef	=> undef,
							poweroffplayers			=> $g{prefs}->bPowerOffPlayers,
							gotosn					=> 0,
							checkblock				=> 1,
							cancelwait				=> 0,
							cmdwait					=> 5,
							refreshwait				=> -1,
							stopsc					=> 0,
							setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
							command					=> $g{prefs}->szEODWatchdog_cmd } ) ;
		} else {
			$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
		}

		if (!defined($item)) {
			$g{log}->error( "BadEODWatchdog Action: $action");
			return;
		}

		#restart the countdown for eventual return from suspend/hibernate/SN..
		$nEODPlayersTimeCount = 0;

		#cue up the requested action..
		$g{tPendingActionTimer} = Slim::Utils::Timers::setTimer( undef, time() + $nWatchdogActionDelay, \&Plugins::SrvrPowerCtrl::Plugin::performAction, ( $item ) );

	} else {
		$g{log}->debug( $g{prefs}->nEODWatchdogTimeout - $nEODPlayersTimeCount . " minutes left..");
	}

}

my $nLastOnWakeupTimeCheck = 0;

sub ActivateOnWakeupWatchdog {
	my $bEnable = shift;

	$nLastOnWakeupTimeCheck = $bEnable ? time() : 0;

	return $bEnable;
}


sub OnWakeupWatchdog {
	my $nCurTime = shift;
	my $szCommand;

	#have we returned from a suspend or hibernation or has system time changed??
	if ( ($nCurTime > ($nLastOnWakeupTimeCheck + 90)) || $nCurTime < $nLastOnWakeupTimeCheck ) {
		$g{log}->info("Wakeup Call!!");

		#reset the OnIdle counts..
		$nIdlePlayersTimeCount = 0;
		$nEODPlayersTimeCount = 0;

		#perform our OnWakup cmd..
		$szCommand = $g{prefs}->szOnWakup_cmd;

		Plugins::SrvrPowerCtrl::Alarms::ExecRTCWakeForAlarmCmd($szCommand, $nCurTime);

		# Pull back any players from SqueezeNetwork that we want..
		if ($g{prefs}->bRetrievePlayersFromSN) {
			Slim::Utils::Timers::setTimer(undef, time() + $g{prefs}->nRetrievePlayersFromSNWait, \&Plugins::SrvrPowerCtrl::Util::PullFromSqueezeNetwork, );
		}
	} #else {
	#	$g{log}->debug("But I haven't been asleep!");
	#}

	$nLastOnWakeupTimeCheck = $nCurTime;

}

my $bSleepTimerIsActive = 0;

sub MonitorSleepRequests {
	my $request = shift;
	my $client;
	my $cmd;
	my $nSleepTime;
	my $nCurrentSleepTime;

	$cmd = $request->getRequestString();

	if ($cmd ne 'sleep') {
		return 0;
	}

	$client = $request->client() || return 0;

	$nSleepTime = $client->sleepTime();
	$nCurrentSleepTime = $client->currentSleepTime();

	$g{log}->debug("MonitorSleepRequests called for client: ". $client->name() . ", cmd: $cmd, sleepTime: $nSleepTime, currentSleepTime: $nCurrentSleepTime");

	#if $nSleepTime || $nCurrentSleepTime are 0, the the user has canceled sleep..and we should cancel end of sleep action..
	if (!$nSleepTime || !$nCurrentSleepTime){
		$g{log}->debug("Sleep timer canceled!");
		$bSleepTimerIsActive = 0;
	} else {
		$g{log}->debug("Sleep timer detected!");
		$bSleepTimerIsActive = -1;
	}

}

sub ActivateSleepEndWatchdog {
	my $bEnable = shift;

	$bSleepTimerIsActive = 0;

	if ($bEnable){
		Slim::Control::Request::subscribe( \&MonitorSleepRequests, [['sleep']]);

	} else {
		Slim::Control::Request::unsubscribe( \&MonitorSleepRequests, [['sleep']]);
	}

	return $bEnable;
}

sub SleepEndWatchdog {
	my $nCurTime = shift;
	my $action;
	my $item;
	my $nSleepTime = Plugins::SrvrPowerCtrl::SleepButton::GetSleepTime();

	if ($nSleepTime && !$bSleepTimerIsActive) {
		$bSleepTimerIsActive = 1;
		$g{log}->debug("Hey!  This shouldn't happen!");
		return 0;
	}

	if (!$nSleepTime && $bSleepTimerIsActive) {
		#we've transitioned to end of sleep..
		$g{log}->debug("End of sleep timer detected!");

		#restart the sleep timer active flag for our eventual return from suspend/hibernate/SN..
		$bSleepTimerIsActive = 0;

		#check to see that all other players are idle before taking action...
		if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersPlaying() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
			$g{log}->debug("Not idle condition..end-of-sleep action not taken!");
			return 0;
		}

		$action = $g{prefs}->szSleepEndWatchdogAction;

		$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
		if (!defined($item)) {
			$g{log}->error( "Bad SleepEndWatchdog Action: $action");
			return;
		}

		#cue up the requested action..
		$g{log}->error( "SleepEndWatchdog Action: $action");
		$g{tPendingActionTimer} = Slim::Utils::Timers::setTimer( undef, time() + $nWatchdogActionDelay, \&Plugins::SrvrPowerCtrl::Plugin::performAction, ( $item ) );
	}
}

my $nTimerInterval = 60;		#number of seconds between watchdog checks..should be 60.

sub ActivateSPCWatchdog {
	my $bEnable = shift;
	my $szTimerName = 'srvrpowerctrl.watchdog';
	my $szParamValue = 'multi.check';

	if ($bEnable) {
		$g{tSPCWatchdogTimer} = Slim::Utils::Timers::setTimer($szTimerName, time() + $nTimerInterval, \&SPCWatchdog, ($szParamValue), );
	} else {
		if (defined($g{tSPCWatchdogTimer})) {
			Slim::Utils::Timers::killSpecific($g{tSPCWatchdogTimer});
			$g{tSPCWatchdogTimer} = 0;
		}

		if (defined($g{tPendingActionTimer})) {
			Slim::Utils::Timers::killSpecific($g{tPendingActionTimer});
			$g{tPendingActionTimer} = 0;
		}
	}
}

#This is the main timer routine that checks the various watchdog conditions..
sub SPCWatchdog {
	my ($szTimerName, $param1, $param2, $param3) = @_;
	my $nCurTime = time();
	my $bIsInEOD = IsInEOD($nCurTime);
	my $szCommand;

	$g{log}->debug("$szTimerName, $param1");

	#Only check watchdogs if we aren't waiting for a cancel action..
	if (!$g{tPendingActionTimer}) {
		#Check our OnWakekup conditions..
		if ( $g{prefs}->bUseOnWakeupWatchdog ) {
			OnWakeupWatchdog($nCurTime);
		}

		#Check the SleepDone watchdog..
		if ( $g{prefs}->bUseSleepEndWatchdog ) {
			SleepEndWatchdog($nCurTime);
		}

		#Check the End-of-day watchdog..
		if ( $bIsInEOD ) {
			#if we aren't skipping this check..
			if ( !$bSkipEODCheck ) {
				EODWatchdog($nCurTime);
			}
		} else {
			#reset the skip flag for next EOD
			$bSkipEODCheck = 0;
			#Check the OnIdle watchdog..
			if ( $g{prefs}->bUseIdleWatchdog  ) {
				IdleWatchdog($nCurTime);
			}
		}
	}

	#Schedule next check..
	$g{tWatchdogTimer} = Slim::Utils::Timers::setTimer($szTimerName, $nCurTime + $nTimerInterval, \&SPCWatchdog, ($param1), );
}

1;
